/*
rvoy Version 1.074
Finding temporary files to produce remove_e.txt
Copyright 2017 Sogaya
Please enter "rvoy -V" for usage.

Watcom

*/
#define VERSION 1.074

#define CONTROL_CARET_CAPITALS_TEXT "CCC*.txt"
#define DIR_BATCH_FILE "DIR*.bat"
#define LS_BATCH_FILE "LS_*.bat"
#define TIME_NAME_CMB_TEXT "cmb*.txt"
#define TIME_NAME_ERROR_TEXT "err*.txt"
#define TIME_NAME_FILE_NAME_LIST_TEXT "FNL*.txt"
#define TIME_NAME_PIC_TEXT "pic*.txt"
#define TIME_NAME_SEE_TEXT "see*.txt"
#define TIME_NAME_TABLE_CSV "tab*.csv"
#define TIME_NAME_TABLE_TXT "tab*.txt"
#define TIME_NAME_TEMPORARY "*.tmp"
#define TIME_NAME_TRUE_TEXT "tru*.txt"
#define TIME_NAME_UNDO_BIN "und*.bin"
#define TIME_NAME_UNIQUE_TXT "unq*.txt"
#define TIME_NAME_WHO_CSV "who*.csv"
#define TIME_NAME_WHO_TEXT "who*.txt"

#define BUFFER_SIZE 256
#define LOWER_LIMIT 64
#define MANY 30
#define MINUS_X_SIZE 2
#define REMOVE_E_TEXT "remove_e.txt"
#define TRUE 1
#include <io.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

typedef int BOOL;

void chopped(char *s) {
    char *p;

/* Removing the new-line character */
    p = strrchr(s, '\n');
    if(p != NULL) {
        *p = '\0';
    }
}

int main(int argc, char *argv[]) {
    BOOL to_be_added = 0; /* FALSE */
    BOOL exclude_large_files = 0; /* FALSE */
    char *p;
    char *stop;
    char c[BUFFER_SIZE];
    char file_type[MANY][BUFFER_SIZE];
    char minus_x[BUFFER_SIZE];
    FILE *file;
    FILE *expression;
    int file_type_count, found, line, minus_x_size;
    long handle;
    struct _finddata_t fileinfo;

    memset(file_type, 0, MANY * BUFFER_SIZE );

    switch(argc) {
    case 2:
        if(! strcmp(argv[1], "-V") ) {
/* rvoy user's guide */
            fprintf(stderr, "rvoy Version %2.3f\n", VERSION);
            fprintf(stderr, "\n");
            fprintf(stderr, "rvoy -V\n");
            fprintf(stderr, "rvoy -x%d\n", LOWER_LIMIT);
            fprintf(stderr, "rvoy  \n");
            fprintf(stderr, "rvoy rvoy.txt\n");
            return 1;
        }

        strcpy(minus_x, argv[1]);
        minus_x[MINUS_X_SIZE] = '\0';
        if(! strcmp(minus_x, "-x") ) {
            minus_x_size = (int)strtol( argv[1] + MINUS_X_SIZE, &stop, 10);
            if(minus_x_size <= 0) {
                minus_x_size = LOWER_LIMIT;
            }

/* Even if found, large files must not be removed. */
            exclude_large_files = TRUE;
            strcpy( & file_type[0][0],  CONTROL_CARET_CAPITALS_TEXT);
            strcpy( & file_type[1][0],  DIR_BATCH_FILE);
            strcpy( & file_type[2][0],  LS_BATCH_FILE);
            strcpy( & file_type[3][0],  TIME_NAME_CMB_TEXT);
            strcpy( & file_type[4][0],  TIME_NAME_ERROR_TEXT);
            strcpy( & file_type[5][0],  TIME_NAME_FILE_NAME_LIST_TEXT);
            strcpy( & file_type[6][0],  TIME_NAME_PIC_TEXT);
            strcpy( & file_type[7][0],  TIME_NAME_SEE_TEXT);
            strcpy( & file_type[8][0],  TIME_NAME_TABLE_CSV);
            strcpy( & file_type[9][0],  TIME_NAME_TABLE_TXT);
            strcpy( & file_type[10][0], TIME_NAME_TEMPORARY);
            strcpy( & file_type[11][0], TIME_NAME_TRUE_TEXT);
            strcpy( & file_type[12][0], TIME_NAME_UNDO_BIN);
            strcpy( & file_type[13][0], TIME_NAME_UNIQUE_TXT);
            strcpy( & file_type[14][0], TIME_NAME_WHO_CSV);
            strcpy( & file_type[15][0], TIME_NAME_WHO_TEXT);
            file_type_count = MANY;
            break;
        } else {
/* Reading rvoy.txt to produce an array of file types. */
            expression = fopen(argv[1], "rt");
            if(expression == NULL) {
                fprintf(stderr, "%s\n", argv[1]);
                return 1;
            }

            for(line = 0; line < MANY; line++) {
                p = fgets(c, BUFFER_SIZE, expression);
                if(p == NULL) {
                    break;
                }

                chopped(c);
                strcpy(&file_type[line][0], c);
            }

            file_type_count = line;
            fclose(expression);
        }

        break;
    default:
        strcpy( & file_type[0][0],  CONTROL_CARET_CAPITALS_TEXT);
        strcpy( & file_type[1][0],  DIR_BATCH_FILE);
        strcpy( & file_type[2][0],  LS_BATCH_FILE);
        strcpy( & file_type[3][0],  TIME_NAME_CMB_TEXT);
        strcpy( & file_type[4][0],  TIME_NAME_ERROR_TEXT);
        strcpy( & file_type[5][0],  TIME_NAME_FILE_NAME_LIST_TEXT);
        strcpy( & file_type[6][0],  TIME_NAME_PIC_TEXT);
        strcpy( & file_type[7][0],  TIME_NAME_SEE_TEXT);
        strcpy( & file_type[8][0],  TIME_NAME_TABLE_CSV);
        strcpy( & file_type[9][0],  TIME_NAME_TABLE_TXT);
        strcpy( & file_type[10][0], TIME_NAME_TEMPORARY);
        strcpy( & file_type[11][0], TIME_NAME_TRUE_TEXT);
        strcpy( & file_type[12][0], TIME_NAME_UNDO_BIN);
        strcpy( & file_type[13][0], TIME_NAME_UNIQUE_TXT);
        strcpy( & file_type[14][0], TIME_NAME_WHO_CSV);
        strcpy( & file_type[15][0], TIME_NAME_WHO_TEXT);
        file_type_count = MANY;
        break;
    }

    file = fopen(REMOVE_E_TEXT, "wt");
    for(line = 0; line < file_type_count; line++) {
        p = & file_type[line][0];
        if(*p == '\0') {
            break;
        }

        handle = _findfirst(p, &fileinfo);
        found = (int)handle;
        while(found != -1) {
            to_be_added = 0; /* FALSE */
            if(exclude_large_files) {
                if(fileinfo.size < minus_x_size) {
                    to_be_added = TRUE;
                }

            } else {
                to_be_added = TRUE;
            }

            if(to_be_added) {
                fprintf(file, "%s\n", fileinfo.name);
            }

            found = _findnext(handle, &fileinfo);
        }

        _findclose(handle);
    }

    fclose(file);
    return 0;
}
